<?php
/***************************************************************************
 *                                                                          *
 *   (c) 2004 Vladimir V. Kalynyak, Alexey V. Vinokurov, Ilya M. Shalnev    *
 *                                                                          *
 * This  is  commercial  software,  only  users  who have purchased a valid *
 * license  and  accept  to the terms of the  License Agreement can install *
 * and use this program.                                                    *
 *                                                                          *
 ****************************************************************************
 * PLEASE READ THE FULL TEXT  OF THE SOFTWARE  LICENSE   AGREEMENT  IN  THE *
 * "copyright.txt" FILE PROVIDED WITH THIS DISTRIBUTION PACKAGE.            *
 ****************************************************************************/


namespace Tygh\Addons\SalesReport\Documents\SalesReport;

use Tygh\Database\Connection;
use Tygh\Exceptions\InputException;
use Tygh\Template\Document\Document;
use Tygh\Template\Document\IIncludableType;
use Tygh\Template\Document\IPreviewableType;
use Tygh\Template\Document\IType;
use Tygh\Template\Document\Repository;
use Tygh\Template\ITemplate;
use Tygh\Template\Renderer;
use Tygh\Template\VariableCollectionFactory;

/**
 * Class Type
 * @package Tygh\Addons\GiftCertifications\Documents\GiftCertificate
 */
class Type implements IType, IPreviewableType, IIncludableType
{
    const DOCUMENT_TYPE = 'default';

    /** @var Repository */
    protected $repository;

    /** @var VariableCollectionFactory */
    protected $collection_factory;

    /** @var Connection */
    protected $connection;

    /** @var Renderer */
    protected $renderer;

    /**
     * Gift certificate document type constructor.
     *
     * @param Repository                        $repository            Instance of document repository.
     * @param Connection                        $connection            Instance of database connection.
     * @param Renderer                          $renderer              Instance of template renderer.
     * @param VariableCollectionFactory         $collection_factory    Instance of variable collection factory.
     */
    public function __construct(
        Repository $repository,
        Connection $connection,
        Renderer $renderer,
        VariableCollectionFactory $collection_factory
    )
    {
        $this->connection = $connection;
        $this->renderer = $renderer;
        $this->repository = $repository;
        $this->collection_factory = $collection_factory;
    }

    /**
     * @inheritDoc
     */
    public function preview(Document $document)
    {
        $condition = fn_get_report_company_condition('?:report.company_id');
        $report_id = (int) $this->connection->getField('SELECT MIN(report_id) FROM ?:report WHERE 1 ?p', $condition);

        if (empty($report_id)) {
            throw new InputException(__("document_preview_report_not_found"));
        }

        $report_data = fn_get_report_info($report_id);

        return $this->render($report_data, $document, DESCR_SL);
    }

    /**
     * @inheritDoc
     */
    public function includeDocument($code, $lang_code, $params)
    {
        $report_id = (int) array_shift($params);

        return $this->renderById($report_id, $code, $lang_code);
    }

    /**
     * @inheritDoc
     */
    public function getCode()
    {
        return self::DOCUMENT_TYPE;
    }

    /**
     * Render gift certificate.
     *
     * @param array     $report   Report data.
     * @param string    $code               Document code.
     * @param string    $lang_code          Language code.
     *
     * @return string
     */
    public function renderByData($report, $code = 'default', $lang_code)
    {
        $document = $this->repository->findByTypeAndCode($this->getCode(), $code);

        if (!$document && $code != 'default') {
            $document = $this->repository->findByTypeAndCode($this->getCode(), 'default');
        }

        if ($document) {
            return $this->render($report, $document, $lang_code);
        }

        return '';
    }

    /**
     * Render gift certificate.
     *
     * @param int       $report_id    Gift certificate identifier.
     * @param string    $code                   Document code.
     * @param string    $lang_code              Language code.
     *
     * @return string
     */
    public function renderById($report_id, $code = 'default', $lang_code)
    {
        $report_data = fn_get_report_statistics($report_id);


        if ($report_data) {
            return $this->renderByData($report_data, $code, $lang_code);
        }

        return '';
    }

    /**
     * Render gift certificate document.
     *
     * @param array     $report   Gift certificate data.
     * @param ITemplate $document           Instance of gift certificate document.
     * @param string    $lang_code          Language code.
     *
     * @return string
     */
    public function render($report, ITemplate $document, $lang_code)
    {
        $context = new Context($report, $lang_code);
        $variable_collection = $this->collection_factory->createCollection(self::SCHEMA_DIR, $this->getCode(), $context);

        return $this->renderer->renderTemplate($document, $context, $variable_collection);
    }
}